# $Id: graph.tcl,v 1.2 2000/05/17 01:12:27 soul Exp $

namespace eval tickergraph {
	namespace export showGraph addData createGraph tickFormat

	option add *Pixels 1
}

proc ::tickergraph::showGraph {symbol} {
	if {[string length [info procs ::tickergraph::${symbol}::showGraph]] == 0} {
		createGraph $symbol
	}
	${symbol}::showGraph
}	

proc ::tickergraph::addData {symbol time value} {
	::tickergraph::${symbol}::addData $time $value
}

proc ::tickergraph::tickFormat {widget value} {
	set value [expr {$value+[clock scan "9:30"]}]
	clock format $value -format "%H:%M"
}

proc ::tickergraph::createGraph {symbol} {
	namespace eval ::tickergraph::$symbol {
		# data points
		variable graphXData
		variable graphYData
		# window containing the graph
		variable graphWindow
		# Da graph
		variable graphWidget
		variable startTime [clock scan "9:30"]
		variable endTime [clock scan "17:00"]
		
		namespace export addData showGraph
		#package require emu_graph
		#namespace import emu_graph::*

		::blt::vector graphXData
		::blt::vector graphYData
		set graphWindow ""
		set graphWidget ""
	} 
		
	proc ::tickergraph::${symbol}::init {} {
		variable graphXData
		variable graphYData
		variable graphWidget
		variable graphWindow
		
		set symbol [namespace tail [namespace current]]
		regsub -all {\.} $symbol "_" wsymbol
		set graphWindow [Dialog .tickergraph$wsymbol -title "$symbol Variations" \
			-modal none]
		set subf [$graphWindow getframe] 
		set graphWidget [::blt::graph $subf.g -title "$symbol Variations" \
			-width 640 -height 480 -bg white]

		$graphWidget xaxis configure -min 0 -stepsize 1800 \
			-max [expr {[clock scan "17:00"] - [clock scan "9:30"]}] \
			-command ::tickergraph::tickFormat -rotate 90 
		$graphWidget element create line -xdata graphXData -ydata graphYData 
		$graphWidget legend configure -hide true
		pack $graphWidget -expand true -fill both
		$graphWindow add -text Close \
			-command [namespace code closeWindow] 
	}

	proc ::tickergraph::${symbol}::closeWindow {} {
		variable graphWindow
		
		# trace vdelete graphData w [list updateData]
		$graphWindow withdraw
	}

	proc ::tickergraph::${symbol}::addData {time value} {
		variable graphXData
		variable graphYData
		variable graphWidget
		# Recalculates every time to take care of date changes. Wastes CPU
		# cycles for nothing, though
		variable startTime [clock scan "9:30"]
		variable endTime [clock scan "17:00"]

		# Don't add data after hours
		if {$time > $startTime && $time < $endTime} {
			graphXData append [expr {$time-$startTime}]
			graphYData append $value
		} elseif {[expr {$time-$startTime}] < 0 && [expr {$time-$startTime}] > -900} {
			# 15 minutes before the bell rings, reset all data
			graphXData set {}
			graphYData set {}
			if {[string length $graphWidget]} {
				$graphWidget xaxis configure -min 0 -stepsize 1800 \
					-max [expr {$endTime - $startTime}]
			}
		}
	}

	proc ::tickergraph::${symbol}::showGraph {} {
		variable graphWindow
		
		if {[string length $graphWindow] == 0} {
			init
		}
		$graphWindow draw
	
	}
}
